# coding=utf-8
import logging

from django.contrib.messages.views import SuccessMessageMixin
from django.urls import reverse_lazy
from django.views.generic import CreateView
from django.views.generic import ListView
from django.views.generic import UpdateView

from web.forms.forms import VoucherForm
from web.models import Voucher
from web.models.constants import GLOBAL_STUDY_ID
from . import WrappedView

logger = logging.getLogger(__name__)


class VoucherListView(ListView, WrappedView):
    model = Voucher
    context_object_name = "vouchers"
    template_name = 'vouchers/list.html'


class VoucherCreateView(CreateView, WrappedView):
    form_class = VoucherForm
    model = Voucher

    template_name = "vouchers/add.html"
    success_url = reverse_lazy('web.views.vouchers')
    success_message = "Voucher type created"

    def form_valid(self, form):
        form.instance.study_id = GLOBAL_STUDY_ID
        # noinspection PyUnresolvedReferences
        form.instance.study_subject_id = self.request.GET.get("study_subject_id", -1)
        return super(VoucherCreateView, self).form_valid(form)

    def get_success_url(self, **kwargs):
        # noinspection PyUnresolvedReferences
        return reverse_lazy('web.views.subject_edit', kwargs={'id': self.request.GET.get("study_subject_id", -1)})


class VoucherEditView(SuccessMessageMixin, UpdateView, WrappedView):
    form_class = VoucherForm
    model = Voucher

    success_url = reverse_lazy('web.views.vouchers')
    success_message = "Voucher saved successfully"
    template_name = "vouchers/edit.html"
    context_object_name = "voucher_type"

    def get_success_url(self, **kwargs):
        # noinspection PyUnresolvedReferences
        study_subject_id = Voucher.objects.get(id=self.kwargs['pk']).study_subject.id
        return reverse_lazy('web.views.subject_edit', kwargs={'id': study_subject_id})
